#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<math.h>

#include "parameter.h"

struct DATA{
  int N;
  double *Count;
  double *Wavelength;
};

template<typename T>T *SET_VECTOR(int n);
DATA SET_DATA(int n);
DATA SET_DATA(DATA tmp);
void FREE_DATA(DATA *tmp);
int SORT_CHECK(DATA tmp);
void SORT_DATA(DATA *tmp);
DATA READ_FILE(const char *filename);
void LSM1(double *X,double *Y,double N,double *a,double *b);
DATA CONVERT_RANGE(DATA tmp1,DATA tmp2);
int REJECT_LINE(DATA *tmp,const char *filename,double low_gnuplot,double upp_gnuplot);
DATA EXTRAPOLATION_LOW(DATA tmp1,DATA tmp2);
DATA EXTRAPOLATION_HIGH(DATA tmp1,DATA tmp2);
void OUTPUT_DATA(DATA tmp,const char *filename);
void GNUPOT_DATA(DATA tmp,double low_gnuplot,double upp_gnuplot);
bool YES_NO(const char *ans1,const char *ans2);
void CHECK_SPEC(DATA tmp,double low_gnuplot,double upp_gnuplot,DATA spec);
void GNUPOT2_DATA(DATA tmp1,DATA tmp2,double low_gnuplot,double upp_gnuplot);
DATA ADD_DATA(DATA tmp);

//#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
//#-#-#-#                MAIN                 #-#-#-#
//#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
int main(int argc,char *argv[])
{
  DATA tmpModel,tmpMeasure;

  printf("\n");
  printf("----------start-------------------------------------------\n");
  //入力値の数が4以下ならエラー
  if(argc<4){
    printf("\n入力エラー!!!\n");
    printf("~path~/CALIBRATION 標準星モデルファイル名 標準星測定ファイル名 ターゲットファイル名\n");
    printf("\n出力ファイル名を指定したい場合は\n");
    printf("~path~/CALIBRATION 標準星モデルファイル名 標準星測定ファイル名 ターゲットファイル名 -o 出力ファイル名\n");
    printf("\n-----------end--------------------------------------------\n\n");
    return 0;
  }
  char ModelFile[100],MeasureFile[100],TargetFile[100],OutFile[100];
  strcpy(ModelFile,argv[1]);
  strcpy(MeasureFile,argv[2]);
  strcpy(TargetFile,argv[3]);
  sprintf(OutFile,"answer_%s",argv[3]);//出力ファイル名　デフォルトはTargetファイル名に"answer_"をつけるだけ
  bool jidou = 0;
  //入力値の数が5以上ならオプションをチェック
  if(argc >= 5){
    for(int i=1;i<=argc;i++){
      //"-o" 出力ファイル名の指定
      if(strcmp(argv[i-1],"-o")==0 && i<argc)   sprintf(OutFile,"%s",argv[i]);
      if(strcmp(argv[i-1],"-f")==0) jidou = 1;
    }
  }
  printf("標準星モデルファイル\t => %s\n",ModelFile);
  printf("標準星測定ファイル\t=> %s\n",MeasureFile);
  printf("ターゲットファイル\t => %s\n",TargetFile);
  printf("出力ファイル\t\t=> %s\n",OutFile);
  //モデルスペクトルのファイルを読む
  DATA Model = READ_FILE(ModelFile);
  tmpModel = ADD_DATA(Model);
  FREE_DATA(&Model);
  Model = SET_DATA(tmpModel);
  FREE_DATA(&tmpModel);

  //測定スペクトルのファイルを読む
  DATA Measure = READ_FILE(MeasureFile);

  //Targetスペクトルのファイルを読む
  DATA Target = READ_FILE(TargetFile);
  printf("\n----------------------------------------------------------\n");
  printf("標準星のスペクトルを表示しますか？");
  if(YES_NO("y","n")) GNUPOT2_DATA(Model,Measure,Measure.Wavelength[1],Measure.Wavelength[Measure.N]);

  //モデルスペクトルの吸収線を手動で削除する
  printf("\n----------------------------------------------------------\n");
  printf("標準星のモデルスペクトルに修正を加えますか？");
  if(YES_NO("y","n")) REJECT_LINE(&Model,ModelFile,Measure.Wavelength[1],Measure.Wavelength[Measure.N]);

  //測定スペクトルの吸収線を手動で削除する
  printf("\n----------------------------------------------------------\n");
  printf("標準星の測定したスペクトルに修正を加えますか？");
  if(YES_NO("y","n")) REJECT_LINE(&Measure,MeasureFile,Measure.Wavelength[1],Measure.Wavelength[Measure.N]);

  //Targetスペクトルの波長範囲より、モデルスペクトルや測定スペクトルの波長範囲が狭い場合、適当に外挿する
  if(Model.Wavelength[1] > Target.Wavelength[1]){
    tmpModel = EXTRAPOLATION_LOW(Model,Target);
    FREE_DATA(&Model);
    Model = SET_DATA(tmpModel);
    FREE_DATA(&tmpModel);
  }
  if(Model.Wavelength[Model.N] < Target.Wavelength[Target.N]){
    tmpModel = EXTRAPOLATION_HIGH(Model,Target);
    FREE_DATA(&Model);
    Model = SET_DATA(tmpModel);
    FREE_DATA(&tmpModel);
  }
  if(Measure.Wavelength[1] > Target.Wavelength[1]){
    tmpMeasure = EXTRAPOLATION_LOW(Measure,Target);
    FREE_DATA(&Measure);
    Measure = SET_DATA(tmpMeasure);
    FREE_DATA(&tmpMeasure);
  }
  if(Measure.Wavelength[Measure.N] < Target.Wavelength[Target.N]){
    tmpMeasure = EXTRAPOLATION_HIGH(Measure,Target);
    FREE_DATA(&Measure);
    Measure = SET_DATA(tmpMeasure);
    FREE_DATA(&tmpMeasure);
  }

  //モデルスペクトルをTargetスペクトルの波長に変換する
  tmpModel = CONVERT_RANGE(Model,Target);
  FREE_DATA(&Model);
  //測定スペクトルをTargetスペクトルの波長に変換する
  tmpMeasure = CONVERT_RANGE(Measure,Target);
  FREE_DATA(&Measure);

  //波長感度曲線の計算
  printf("\n----------------------------------------------------------\n");
  printf("感度曲線を計算します\n");
  DATA Factor = SET_DATA(Target.N);
  DATA Coll_Target = SET_DATA(Target.N);
  for(int i=1;i<=Target.N;i++){
    Factor.Wavelength[i] = Target.Wavelength[i];
    Factor.Count[i] = tmpModel.Count[i] / tmpMeasure.Count[i];
    Coll_Target.Wavelength[i] = Target.Wavelength[i];
    Coll_Target.Count[i] = Factor.Count[i] * Target.Count[i];
  }
  FREE_DATA(&tmpModel);
  FREE_DATA(&tmpMeasure);

  //結果の出力
  printf("\n----------------------------------------------------------\n");
  printf("結果を出力します\n");
  printf("    => %s\n",OutFile);
  FILE *fout;
  fout = fopen(OutFile,"w");
  fprintf(fout,"# $1 = Wavelength #\n");
  fprintf(fout,"# $2 = Target Count #\n");
  fprintf(fout,"# $3 = Factor #\n");
  if(!jidou) fprintf(fout,"# $2 x $3 = Target intrinsic spectrum #\n");
  else if(jidou){
    fprintf(fout,"# $4 = Target intrinsic spectrum #\n");
    fprintf(fout,"# $2 x $3 = $4 #\n");
  }
  for(int i=1;i<=Target.N;i++){
    if(!jidou)fprintf(fout,"%8lf\t%8lf\t%.5le\n",Target.Wavelength[i],Target.Count[i],Factor.Count[i]);
    else fprintf(fout,"%8lf\t%8lf\t%.5le\t%.5le\n",Target.Wavelength[i],Target.Count[i],Factor.Count[i],Coll_Target.Count[i]);
  }
  fclose(fout);

  printf("\n----------------------------------------------------------\n");
  printf("波長感度曲線を表示しますか？");
  if(YES_NO("y","n")) GNUPOT_DATA(Factor,Factor.Wavelength[1],Factor.Wavelength[Factor.N]);
  if(jidou){
    printf("補正したターゲットのスペクトルを表示しますか？");
    if(YES_NO("y","n")) GNUPOT_DATA(Coll_Target,Factor.Wavelength[1],Factor.Wavelength[Factor.N]);
  }
  FREE_DATA(&Target);
  FREE_DATA(&Factor);
  FREE_DATA(&Coll_Target);
  printf("\n-----------end--------------------------------------------\n\n");

  return 0;
}





//##################################################
//######             SET_VECTOR               ######
//##################################################
template<typename T>T *SET_VECTOR(int n)
{
  T *v;
  v = (T *)malloc((size_t)((n + 1) * sizeof(T)));
  if(v == NULL){
    printf("GET_VECTOR : ERROR!!!!\n");
    printf("\n-----------end--------------------------------------------\n\n");
    exit(1);
  }
  //initialize
  for(int i=0;i<=n;i++) v[i]=0;
  return v; 
}
template int *SET_VECTOR<int>(int n);
template double *SET_VECTOR<double>(int n);

//##################################################
//######              SET_DATA                ######
//##################################################
DATA SET_DATA(int n)
{
  DATA tmp;
  tmp.N = n;
  tmp.Count = SET_VECTOR<double>(n);
  tmp.Wavelength = SET_VECTOR<double>(n);
  return tmp;
}
DATA SET_DATA(DATA tmp)
{
  DATA ans = SET_DATA(tmp.N);
  for(int i=1;i<=ans.N;i++){
    ans.Count[i] = tmp.Count[i];
    ans.Wavelength[i] = tmp.Wavelength[i];
  }
  return ans;
}

//##################################################
//######              FREE_DATA               ######
//##################################################
void FREE_DATA(DATA *tmp)
{
  tmp->N = 0;
  free(tmp->Wavelength);
  tmp->Wavelength = NULL;
  free(tmp->Count);
  tmp->Count = NULL;
}

//##################################################
//######              SORT_CHECK              ######
//##################################################
int SORT_CHECK(DATA tmp)
{
  for(int i=1;i<=tmp.N-1;i++) if(tmp.Wavelength[i] > tmp.Wavelength[i+1]) return -1;
  return 0;
}

//##################################################
//######               SORT_DATA              ######
//##################################################
void SORT_DATA(DATA *tmp)
{
  long l,j,ir,i;
  double rrb,rra;
  l = (tmp->N >> 1) + 1;
  ir = tmp->N;
  for(;;){
    if(l > 1){
      rra = tmp->Wavelength[--l];
      rrb = tmp->Count[l];
    }
    else{
      rra = tmp->Wavelength[ir];
      rrb = tmp->Count[ir];
      tmp->Wavelength[ir] = tmp->Wavelength[1];
      tmp->Count[ir] = tmp->Count[1];
      if(--ir == 1){
	tmp->Wavelength[1] = rra;
	tmp->Count[1] = rrb;
	return;
      }
    }
    i = l;
    j = l << 1;
    while(j <= ir){
      if(j < ir && tmp->Wavelength[j] < tmp->Wavelength[j+1]) ++j;
      if(rra < tmp->Wavelength[j]){
	tmp->Wavelength[i] = tmp->Wavelength[j];
	tmp->Count[i] = tmp->Count[j];
	j += (i=j);
      }
      else j = ir + 1;
    }
    tmp->Wavelength[i] = rra;
    tmp->Count[i] = rrb;
  }
}

//##################################################
//######              READ_FILE               ######
//##################################################
DATA READ_FILE(const char *filename)
{
  double count,wavelength;
  FILE *fp;
  //count number of row in this file
  int n = 0;
  if((fp=fopen(filename,"r"))==NULL){
    printf("\nファイル読み込みエラー!!!! [%s]が読み込めません\n",filename);
    printf("プログラムを終了します\n");
    printf("\n-----------end--------------------------------------------\n\n");
    exit(1);
  }
  while(fscanf(fp,"%le %le",&count,&wavelength)!=EOF) n++;
  fclose(fp);
  //set DATA
  DATA tmp = SET_DATA(n);
  //read the data in this file
  if((fp=fopen(filename,"r"))==NULL){
    printf("\nファイル読み込みエラー!!!! [%s]が読み込めません\n",filename);
    printf("プログラムを終了します\n");
    printf("\n-----------end--------------------------------------------\n\n");
    exit(1);
  }
  n=0;
  while(fscanf(fp,"%le %le",&wavelength,&count)!=EOF){
    n++;
    tmp.Count[n] = count;
    tmp.Wavelength[n] = wavelength;
  }
  if(SORT_CHECK(tmp)==-1) SORT_DATA(&tmp);
  return tmp;
}

//##################################################
//######            CONVERT_RANGE             ######
//##################################################
DATA CONVERT_RANGE(DATA tmp1,DATA tmp2)
{
  //tmp1 -> tmp2
  DATA ans = SET_DATA(tmp2.N);
  double min,dx,d;
  int minj=1;
  for(int i=1;i<=tmp2.N;i++){
    ans.Wavelength[i] = tmp2.Wavelength[i];
    min = 999999;
    for(int j=minj;j<=tmp1.N;j++){
      dx = ans.Wavelength[i] - tmp1.Wavelength[j];
      if(dx < 0) break;
      else if(dx >= 0 && dx < min){
	min = dx;
	minj = j;
      }
    }
    d = tmp1.Wavelength[minj+1] - tmp1.Wavelength[minj];
    ans.Count[i] = tmp1.Count[minj] * (1 - min / d) + tmp1.Count[minj + 1] * min / d;
  }
  return ans;
}

//##################################################
//######                 LSM1                 ######
//##################################################
void LSM1(double *X,double *Y,int N,double *a,double *b)
{
  double sumx = 0.,sumy = 0.,sumxy = 0.,sumxx = 0.;
  for(long i=1;i<=N;i++){
    sumx += X[i];
    sumy += Y[i];
    sumxy += X[i]*Y[i];
    sumxx += X[i]*X[i];
  }
  *a=(N*sumxy-sumx*sumy)/(N*sumxx-sumx*sumx);
  *b=(sumxx*sumy-sumxy*sumx)/(N*sumxx-sumx*sumx);
}

//##################################################
//######                 LSM2                 ######
//##################################################
void LSM2(double *X,double *Y,int N,double *a,double *b,double *c)
{
  double x = 0.,y = 0.,xy = 0.,xx = 0.,xxx = 0.,xxxx = 0.,xxy = 0;
  for(long i=1;i<=N;i++){
    x += X[i];
    y += Y[i];
    xy += X[i]*Y[i];
    xx += X[i]*X[i];
    xxx += X[i]*X[i]*X[i];
    xxxx += X[i]*X[i]*X[i]*X[i];
    xxy += X[i]*X[i]*Y[i];
  }
  double denominator = 2 * x * xx * xxx + N * xx * xxxx - x * x * xxxx - N * xxx * xxx - xx * xx * xx;
  *a = (N * xx * xxy - x * x * xxy + x * xx * xy - N * xxx * xy + x * xxx * y - xx * xx * y) / denominator;
  *b = (x * xx * xxy - N * xxx * xxy + N * xxxx * xy - xx * xx * xy + xx * xxx * y - x * xxxx * y) / denominator;
  *c = (-xx * xx * xxy + x * xxx * xxy - x * xxxx * xy + xx * xxx * xy - xxx * xxx * y + xx * xxxx * y) / denominator;
}

//##################################################
//######             REJECT_LINE              ######
//##################################################
int REJECT_LINE(DATA *tmp,const char *filename,double low_gnuplot,double upp_gnuplot)
{

  char input[100];
  double rlow,rupp,est,elow,eupp,a,b,c;
  DATA tmp_spec;
  int *rem;
  int fit;
  int n;
  int nn;
  printf("\n");
  while(1){
    printf("スペクトルを表示しますか？");
    if(YES_NO("y","n")) GNUPOT_DATA(*tmp,low_gnuplot,upp_gnuplot);

    //補正する範囲を指定
  again1: ;
    printf("\n取り除きたい波長範囲を入力してください\n");
    printf("下限値は？ : ");
    scanf("%lf",&rlow);
    printf("上限値は？ : ");
    scanf("%lf",&rupp);
    printf("\n");
    if(rupp < rlow){
      double swap = rupp;
      rupp = rlow;
      rlow = swap;
    }
    //選んだ範囲が、データ点の外側ではないかチェック
    if(rlow < tmp->Wavelength[1] || tmp->Wavelength[tmp->N] < rlow || rupp < tmp->Wavelength[1] || tmp->Wavelength[tmp->N] < rupp){
      printf("エラー!!! 入力した値は範囲外です。n");
      printf("もう一度、入力してください\n");
      goto again1;
    }
    //範囲内にデータ点があるかチェック
    n=0;
    for(int i=1;i<=tmp->N;i++){
      double X = tmp->Wavelength[i];
      if(rlow < X && X < rupp) n++;
    }
    if(n == 0){
      printf("エラー!!! 範囲内に一致する点がありません!!!\n");
      printf("もう一度、入力してください\n");
      goto again1;
    }
    nn = n;
    //補間するための連続成分の見積もり方を選択
    //auto => 補正範囲の外側の最も近い2点を通るような直線で補間する
    //fit1 => 補正範囲外かつ見積もる範囲内の点から、1次の最小二乗法で求めた直線で補間
    //fit2 => 補正範囲外かつ見積もる範囲内の点から、2次の最小二乗法で求めた曲線で補間
    printf("連続成分はどのように見積もりますか？\n");
    printf("auto => 自動で1次　, fit1 => 範囲選択して1次 ,　fit2 => 範囲選択して2次\n");
    printf("(auto or fit1  or fit2) : ");
  again_est: ;
    scanf("%s",input);
    //auto mode
    if(strcmp(input,"auto")==0){
      //calculate continuum
      bool flag = 0;
      double X1,X2,Y1,Y2;
      for(int i=1;i<=tmp->N;i++){
	if(rlow <tmp->Wavelength[i] && !flag){
	  X1 = tmp->Wavelength[i-1];
	  Y1 = tmp->Count[i-1];
	  flag = 1;
	}
	if(rupp < tmp->Wavelength[i] && flag){
	  X2 = tmp->Wavelength[i];
	  Y2 = tmp->Count[i];
	  break;
	}
      }
      a = (Y1-Y2)/(X1-X2);
      b = (Y2*X1-Y1*X2)/(X1-X2);
      fit = 1;
    }
    //fit1 & fit2 mode
    else if(strcmp(input,"fit1")==0 || strcmp(input,"fit2")==0){
      if(strcmp(input,"fit1")==0) fit = 1;
      else fit = 2;
    again2: ;
      printf("削除する範囲からどのくらい広い範囲で連続成分を見積もりますか ？ : +- "); 
      scanf("%lf",&est);
      printf("\n");
      elow = rlow -est;
      eupp = rupp + est;
      //選んだ範囲が、データ外ではないかチェック
      if(elow < tmp->Wavelength[1] || tmp->Wavelength[tmp->N] < elow || eupp < tmp->Wavelength[1] || tmp->Wavelength[tmp->N] < eupp){
	printf("エラー!!! 入力した値は範囲外です。n");
	printf("もう一度、入力してください\n");
	goto again2;
      }
      //選んだ範囲内にデータ点があるかチェック
      n=0;
      for(int i=1;i<=tmp->N;i++){
	double X = tmp->Wavelength[i];
	if(rlow < X && X < rupp) continue;
	else if(elow < X && X < eupp) n++;
      }
      if(n == 0){
	printf("エラー!!! 範囲内に一致する点がありません!!!\n");
	printf("もう一度、入力してください\n");
	goto again2;
      }

      double *tX = SET_VECTOR<double>(tmp->N);
      double *tY = SET_VECTOR<double>(tmp->N);
      n=0;
      for(int i=1;i<=tmp->N;i++){
	double X = tmp->Wavelength[i];
	double Y = tmp->Count[i];
	if(rlow < X && X < rupp) continue;
	else if(elow < X && X < eupp){
	  n++;
	  tX[n] = X;
	  tY[n] = Y;
	}
      }
      if(fit == 1){
	printf("N = %d   1次でフィッティングします\n",n);
	LSM1(tX,tY,n,&a,&b);
      }
      else if(fit == 2){
	printf("N = %d   2次でフィッティングします\n",n);
	LSM2(tX,tY,n,&a,&b,&c);
      }
      free(tX);
      free(tY);
    }
    else{
      printf("エラー!! \"auto\" か \"fit1\" か \"fit2\" を入力してください!!!　(auto or fit1 or fit2) : ");
      goto again_est;
    }
    //求めた直線 or 曲線で補間する
    tmp_spec = SET_DATA(nn);
    rem = SET_VECTOR<int>(nn);
    n=0;
    for(int i=1;i<=tmp->N;i++){
      double X = tmp->Wavelength[i];
      if(rlow < X && X < rupp && fit == 1){
	n++;
	tmp_spec.Wavelength[n] = X;
	tmp_spec.Count[n] = a * X + b;
	rem[n] = i;
      }
      if(rlow < X && X < rupp && fit == 2){
	n++;
	tmp_spec.Wavelength[n] = X;
	tmp_spec.Count[n] = a * X * X + b * X + c;
	rem[n] = i;
      }
    }
    printf("補正結果を確認しますか？");
    bool check = YES_NO("y","n");
    if(check){
      CHECK_SPEC(*tmp,low_gnuplot,upp_gnuplot,tmp_spec);
      printf("これでいいですか？");
      if(YES_NO("y","n")) for(int i=1;i<=tmp_spec.N;i++) tmp->Count[rem[i]] = tmp_spec.Count[i];
    }
    else if(!check) for(int i=1;i<=tmp_spec.N;i++) tmp->Count[rem[i]] = tmp_spec.Count[i];
    FREE_DATA(&tmp_spec);
    free(rem);
    printf("\nさらに補正を加えますか？");
    if(!YES_NO("y","n")) break;
    else if(strcmp(input,"y")==0) ;
  }
  return 0;
}

//##################################################
//######            EXTRAPOLATION             ######
//##################################################
DATA EXTRAPOLATION_LOW(DATA tmp1,DATA tmp2)
{
  //tmp1.Wavelength[1] > tmp2.Wavelength[1]
  int n=0;
  for(int i=1;i<=tmp2.N;i++){
    if(tmp2.Wavelength[i] > tmp1.Wavelength[1]) break;
    n++;
  }
  DATA ans = SET_DATA(tmp1.N+n);
  double a,b;
  LSM1(tmp1.Wavelength,tmp1.Count,10,&a,&b);
  for(int i=1;i<=ans.N;i++){
    if(i<=n){
      ans.Wavelength[i] = tmp2.Wavelength[i];
      ans.Count[i] = a * ans.Wavelength[i] + b;
    }
    else {
      ans.Wavelength[i] = tmp1.Wavelength[i-n];
      ans.Count[i] = tmp1.Count[i-n];
    }
  }
  return ans;
}
DATA EXTRAPOLATION_HIGH(DATA tmp1,DATA tmp2)
{
  //tmp1.Wavelength[tmp1.N] < tmp2.Wavelength[tmp2.N]
  int n=0;
  for(int i=tmp2.N;i>=1;i--){
    if(tmp2.Wavelength[i] < tmp1.Wavelength[tmp1.N]) break;
    n++;
  }
  DATA ans = SET_DATA(tmp1.N+n);
  double a,b;
  double tX[11],tY[11];
  for(int i=0;i<=9;i++){
    tX[i+1] = tmp1.Wavelength[tmp1.N-i];
    tY[i+1] = tmp1.Count[tmp1.N-i];
  }
  LSM1(tX,tY,10,&a,&b);
  for(int i=1;i<=ans.N;i++){
    if(i>tmp1.N){
      ans.Wavelength[i] = tmp2.Wavelength[tmp2.N-n+(i-tmp1.N)];
      ans.Count[i] = a * ans.Wavelength[i] + b;
    }
    else {
      ans.Wavelength[i] = tmp1.Wavelength[i];
      ans.Count[i] = tmp1.Count[i];
    }
  }
  return ans;
}

//##################################################
//######             OUTPUT_DATA              ######
//##################################################
void OUTPUT_DATA(DATA tmp,const char *filename)
{
  FILE *fp;
  fp = fopen(filename,"w");
  fprintf(fp,"# N = %d #\n",tmp.N);
  fprintf(fp,"# $1 = Wavelength\n");
  fprintf(fp,"# $2 = Count\n");
  for(int i=1;i<=tmp.N;i++){
    fprintf(fp,"%8lf\t%8le\n",tmp.Wavelength[i],tmp.Count[i]);
  }
  fclose(fp);
}

//##################################################
//######             GNUPLOT_DATA             ######
//##################################################
void GNUPOT_DATA(DATA tmp,double low_gnuplot,double upp_gnuplot)
{
  FILE *gp;
  gp=popen("gnuplot -persist ","w");
  fprintf(gp,"set xlabel \"wavelength\"\n");
  fprintf(gp,"set ylabel \"count (flux)\"\n");
  fprintf(gp,"set term x11 0\n");
  fprintf(gp,"set xrange[%lf:%lf]\n",low_gnuplot - gnuplot_range,upp_gnuplot + gnuplot_range);
  fprintf(gp,"pl \"-\" w %s\n",gnuplot_linetype);
  for(int i=1;i<=tmp.N;i++)  fprintf(gp,"%le\t%le\n",tmp.Wavelength[i],tmp.Count[i]);
  fprintf(gp,"e\n");
  fprintf(gp,"q\n");
  pclose(gp);
}

//##################################################
//######            GNUPLOT2_DATA             ######
//##################################################
void GNUPOT2_DATA(DATA tmp1,DATA tmp2,double low_gnuplot,double upp_gnuplot)
{
  double max1 = -9999,max2 = -9999;
  for(int i=1;i<=tmp1.N;i++) if(tmp1.Count[i] > max1 && low_gnuplot < tmp1.Wavelength[i] && tmp1.Wavelength[i] < upp_gnuplot) max1 = tmp1.Count[i];
  for(int i=1;i<=tmp2.N;i++) if(tmp2.Count[i] > max2 && low_gnuplot < tmp2.Wavelength[i] && tmp2.Wavelength[i] < upp_gnuplot) max2 = tmp2.Count[i];

  FILE *gp;
  gp=popen("gnuplot -persist ","w");
  fprintf(gp,"set term x11 0\n");
  fprintf(gp,"set title \"Standard Spectrum\"\n");
  fprintf(gp,"set xlabel \"wavelength\"\n");
  fprintf(gp,"set ylabel \"normalized count (flux)\"\n");
  fprintf(gp,"set xrange[%lf:%lf]\n",low_gnuplot - gnuplot_range,upp_gnuplot + gnuplot_range);
  fprintf(gp,"pl \"-\" w %s lc %d ti \"Model Spectram\",\"-\" w %s lc %d ti \"Measure Spectrum\"\n",gnuplot_linetype,gnuplot_color1,gnuplot_linetype,gnuplot_color2);
  for(int i=1;i<=tmp1.N;i++)  fprintf(gp,"%le\t%le\n",tmp1.Wavelength[i],tmp1.Count[i]/max1);
  fprintf(gp,"e\n");
  for(int i=1;i<=tmp2.N;i++)  fprintf(gp,"%le\t%le\n",tmp2.Wavelength[i],tmp2.Count[i]/max2);
  fprintf(gp,"e\n");
  fprintf(gp,"q\n");
  pclose(gp);
}


//##################################################
//######              CHECK_SPEC              ######
//##################################################
void CHECK_SPEC(DATA tmp,double low_gnuplot,double upp_gnuplot,DATA spec)
{
  FILE *gp;
  gp=popen("gnuplot -persist ","w");
  fprintf(gp,"set term x11 0\n");
  fprintf(gp,"set xlabel \"wavelength\"\n");
  fprintf(gp,"set ylabel \"count (flux)\"\n");
  fprintf(gp,"set xrange[%lf:%lf]\n",low_gnuplot - gnuplot_range,upp_gnuplot + gnuplot_range);
  fprintf(gp,"pl \"-\" w %s lc %d,\"-\" w %s lc %d\n",gnuplot_linetype,gnuplot_color1,gnuplot_linetype,gnuplot_color2);
  for(int i=1;i<=tmp.N;i++)  fprintf(gp,"%le\t%le\n",tmp.Wavelength[i],tmp.Count[i]);
  fprintf(gp,"e\n");
  for(int i=1;i<=spec.N;i++)  fprintf(gp,"%le\t%le\n",spec.Wavelength[i],spec.Count[i]);
  fprintf(gp,"e\n");
  fprintf(gp,"q\n");
  pclose(gp);
}

//##################################################
//######                YES_NO                ######
//##################################################
bool YES_NO(const char *ans1,const char *ans2)
{
  char input[100];
  printf("  (%s or %s) : ",ans1,ans2);
 again : ;
  scanf("%s",input);
  if(strcmp(input,ans1)==0) return 1;
  else if(strcmp(input,ans2)==0) return 0;
  else{
    printf("エラー!! \"%s\" か \"%s\" を入力してください!!!　(%s or %s) : ",ans1,ans2,ans1,ans2);
    goto again;
  }
}

//##################################################
//######               ADD_DATA               ######
//##################################################
DATA ADD_DATA(DATA tmp)
{
  DATA ans = SET_DATA(tmp.N * 2 - 1);
  for(int i=1;i<=ans.N;i++){
    if(i % 2 == 1){
      ans.Wavelength[i] = tmp.Wavelength[(i + 1) / 2];
      ans.Count[i] = tmp.Count[(i + 1) / 2];
    }
    else{
      ans.Wavelength[i] = (tmp.Wavelength[i / 2] + tmp.Wavelength[i / 2 + 1]) / 2.;
      ans.Count[i] = (tmp.Count[i / 2] + tmp.Count[i / 2 + 1]) / 2.;
    }
  }
  return ans;
}
